@@ -210,8 +210,7 @@ class BaseInputFilter implements
      */
     public function isValid($context = null)
     {
-        $data = $this->getRawValues();
-        if (null === $data) {
+        if (null === $this->data) {
             throw new Exception\RuntimeException(sprintf(
                 '%s: no data present to validate!',
                 __METHOD__
@@ -219,7 +218,7 @@ class BaseInputFilter implements
         }
 
         $inputs = $this->validationGroup ?: array_keys($this->inputs);
-        return $this->validateInputs($inputs, $data, $context);
+        return $this->validateInputs($inputs, $this->data, $context);
     }
 
     /**
@@ -244,7 +243,24 @@ class BaseInputFilter implements
         foreach ($inputs as $name) {
             $input      = $this->inputs[$name];
 
-            // make sure we have a value (empty) for validation of context
+            // If the value is required, but not present in the data set,
+            // validation fails.
+            if (!array_key_exists($name, $data)
+                && $input instanceof InputInterface
+                && $input->isRequired()
+            ) {
+                $input->setErrorMessage('Value is required');
+                $this->invalidInputs[$name] = $input;
+
+                if ($input->breakOnFailure()) {
+                    return false;
+                }
+
+                $valid = false;
+                continue;
+            }
+
+            // Make sure we have a value (empty) for validation of context
             if (!array_key_exists($name, $data)) {
                 $data[$name] = null;
             }
